@@ -1,7 +1,8 @@
 from __future__ import annotations
 
+from collections.abc import Callable
 from dataclasses import dataclass, field
-from typing import Any, Callable, Self
+from typing import Any, Self
 
 Criteria = Callable[[Any], str]
 
