@@ -1,10 +1,12 @@
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
+from collections.abc import Mapping
 from dataclasses import dataclass, field
 from typing import Any, Self
 
 from fastapi import APIRouter, FastAPI
+from starlette.middleware.cors import CORSMiddleware
 from starlette.responses import JSONResponse
 
 from apexdevkit.error import ApiError
@@ -18,7 +20,7 @@ class FastApiBuilder:
     def build(self) -> FastAPI:
         self.app.add_exception_handler(
             ApiError,
-            lambda request, exc: JSONResponse(content=exc.data, status_code=exc.code),
+            lambda _, exc: JSONResponse(content=exc.data, status_code=exc.code),
         )
         return self.app
 
@@ -44,6 +46,9 @@ class FastApiBuilder:
         return self
 
     def with_route(self, **values: APIRouter) -> Self:
+        return self.with_routes(values)
+
+    def with_routes(self, values: Mapping[str, APIRouter]) -> Self:
         for key, value in values.items():
             self.app.include_router(
                 value, prefix=f"/{key}", tags=value.tags or [key.title()]
@@ -51,6 +56,28 @@ class FastApiBuilder:
 
         return self
 
+    def with_mounted(self, **apps: FastAPI) -> Self:
+        for path, app in apps.items():
+            self.app.mount(f"/{path.replace('_', '-')}", app)
+
+        return self
+
+    def with_frontend(self, origin: str) -> Self:
+        self.app.add_middleware(
+            CORSMiddleware,
+            allow_origins=[origin],
+            allow_credentials=True,
+            allow_methods=["*"],
+            allow_headers=["*"],
+        )
+
+        return self
+
+    def with_swagger(self, **config: Any) -> Self:
+        self.app.swagger_ui_parameters = config
+
+        return self
+
 
 @dataclass
 class RestfulServiceBuilder(ABC):
@@ -70,3 +97,11 @@ class RestfulServiceBuilder(ABC):
     @abstractmethod
     def build(self) -> RestfulService:  # pragma: no cover
         pass
+
+
+@dataclass
+class PreBuilt(RestfulServiceBuilder):  # pragma: no cover
+    service: RestfulService
+
+    def build(self) -> RestfulService:
+        return self.service
