@@ -6,7 +6,6 @@ from typing import Annotated, Any, Protocol, Self, TypeVar
 from fastapi import APIRouter, Depends, Path, Query
 from fastapi.responses import JSONResponse
 
-from apexdevkit.fastapi.builder import RestfulServiceBuilder
 from apexdevkit.fastapi.name import RestfulName
 from apexdevkit.fastapi.resource import RestfulResource
 from apexdevkit.fastapi.response import RestfulResponse
@@ -24,14 +23,6 @@ class Dependency(Protocol):  # pragma: no cover
         pass
 
 
-@dataclass
-class PreBuiltRestfulService(RestfulServiceBuilder):  # pragma: no cover
-    service: RestfulService
-
-    def build(self) -> RestfulService:
-        return self.service
-
-
 @dataclass
 class RestfulRouter:
     router: APIRouter = field(default_factory=APIRouter)
@@ -39,6 +30,8 @@ class RestfulRouter:
     name: RestfulName = field(init=False)
     fields: SchemaFields = field(init=False)
 
+    dependency: Dependency | None = None
+
     @cached_property
     def schema(self) -> RestfulSchema:
         return RestfulSchema(name=self.name, fields=self.fields)
@@ -70,15 +63,20 @@ class RestfulRouter:
 
         return self
 
+    def with_dependency(self, value: Dependency) -> Self:
+        self.dependency = value
+
+        return self
+
     def with_create_one_endpoint(
         self,
-        dependency: Dependency,
+        dependency: Dependency | None = None,
         is_documented: bool = True,
     ) -> Self:
         self.router.add_api_route(
             "",
             self.resource.create_one(
-                Service=dependency.as_dependable(),
+                Service=self._resolve(dependency),
                 Item=Annotated[
                     RawItem,
                     Depends(self.schema.for_create_one()),
@@ -96,13 +94,13 @@ class RestfulRouter:
 
     def with_create_many_endpoint(
         self,
-        dependency: Dependency,
+        dependency: Dependency | None = None,
         is_documented: bool = True,
     ) -> Self:
         self.router.add_api_route(
             "/batch",
             self.resource.create_many(
-                Service=dependency.as_dependable(),
+                Service=self._resolve(dependency),
                 Collection=Annotated[
                     RawCollection,
                     Depends(self.schema.for_create_many()),
@@ -120,13 +118,13 @@ class RestfulRouter:
 
     def with_read_one_endpoint(
         self,
-        dependency: Dependency,
+        dependency: Dependency | None = None,
         is_documented: bool = True,
     ) -> Self:
         self.router.add_api_route(
             self.item_path,
             self.resource.read_one(
-                Service=dependency.as_dependable(),
+                Service=self._resolve(dependency),
                 ItemId=Annotated[
                     str,
                     Path(alias=self.id_alias),
@@ -144,14 +142,14 @@ class RestfulRouter:
 
     def with_read_many_endpoint(
         self,
-        dependency: Dependency,
         query: FluentDict[Any],
+        dependency: Dependency | None = None,
         is_documented: bool = True,
     ) -> Self:
         self.router.add_api_route(
             "",
             self.resource.read_many(
-                Service=dependency.as_dependable(),
+                Service=self._resolve(dependency),
                 QueryParams=Annotated[
                     Schema(self.name).optional_schema_for("ReadMany", query), Query()
                 ],
@@ -166,16 +164,58 @@ class RestfulRouter:
 
         return self
 
+    def with_filter_endpoint(
+        self,
+        dependency: Dependency | None = None,
+        is_documented: bool = True,
+    ) -> Self:
+        self.router.add_api_route(
+            "/filter",
+            self.resource.filter_with(
+                Service=self._resolve(dependency),
+                QueryOptions=Annotated[RawItem, Depends(self.schema.for_filters())],
+            ),
+            methods=["POST"],
+            status_code=200,
+            responses={},
+            response_model=self.schema.for_collection(),
+            include_in_schema=is_documented,
+            summary="Read Filtered",
+        )
+
+        return self
+
+    def with_aggregation_endpoint(
+        self,
+        dependency: Dependency | None = None,
+        is_documented: bool = True,
+    ) -> Self:
+        self.router.add_api_route(
+            "/aggregation",
+            self.resource.aggregation_with(
+                Service=self._resolve(dependency),
+                FilterOptions=Annotated[
+                    RawItem, Depends(self.schema.for_aggregation())
+                ],
+            ),
+            methods=["POST"],
+            status_code=200,
+            responses={},
+            response_model=self.schema.for_aggregation_result(),
+            include_in_schema=is_documented,
+            summary="Aggregation",
+        )
+
+        return self
+
     def with_read_all_endpoint(
         self,
-        dependency: Dependency,
+        dependency: Dependency | None = None,
         is_documented: bool = True,
     ) -> Self:
         self.router.add_api_route(
             "",
-            self.resource.read_all(
-                Service=dependency.as_dependable(),
-            ),
+            self.resource.read_all(Service=self._resolve(dependency)),
             methods=["GET"],
             status_code=200,
             responses={},
@@ -188,13 +228,13 @@ class RestfulRouter:
 
     def with_update_one_endpoint(
         self,
-        dependency: Dependency,
+        dependency: Dependency | None = None,
         is_documented: bool = True,
     ) -> Self:
         self.router.add_api_route(
             self.item_path,
             self.resource.update_one(
-                Service=dependency.as_dependable(),
+                Service=self._resolve(dependency),
                 ItemId=Annotated[
                     str,
                     Path(alias=self.id_alias),
@@ -216,13 +256,13 @@ class RestfulRouter:
 
     def with_update_many_endpoint(
         self,
-        dependency: Dependency,
+        dependency: Dependency | None = None,
         is_documented: bool = True,
     ) -> Self:
         self.router.add_api_route(
             "",
             self.resource.update_many(
-                Service=dependency.as_dependable(),
+                Service=self._resolve(dependency),
                 Collection=Annotated[
                     RawCollection,
                     Depends(self.schema.for_update_many()),
@@ -240,13 +280,13 @@ class RestfulRouter:
 
     def with_replace_one_endpoint(
         self,
-        dependency: Dependency,
+        dependency: Dependency | None = None,
         is_documented: bool = True,
     ) -> Self:
         self.router.add_api_route(
             "",
             self.resource.replace_one(
-                Service=dependency.as_dependable(),
+                Service=self._resolve(dependency),
                 Item=Annotated[
                     RawItem,
                     Depends(self.schema.for_replace_one()),
@@ -264,13 +304,13 @@ class RestfulRouter:
 
     def with_replace_many_endpoint(
         self,
-        dependency: Dependency,
+        dependency: Dependency | None = None,
         is_documented: bool = True,
     ) -> Self:
         self.router.add_api_route(
             "/batch",
             self.resource.replace_many(
-                Service=dependency.as_dependable(),
+                Service=self._resolve(dependency),
                 Collection=Annotated[
                     RawCollection,
                     Depends(self.schema.for_replace_many()),
@@ -288,13 +328,13 @@ class RestfulRouter:
 
     def with_delete_one_endpoint(
         self,
-        dependency: Dependency,
+        dependency: Dependency | None = None,
         is_documented: bool = True,
     ) -> Self:
         self.router.add_api_route(
             self.item_path,
             self.resource.delete_one(
-                Service=dependency.as_dependable(),
+                Service=self._resolve(dependency),
                 ItemId=Annotated[
                     str,
                     Path(alias=self.id_alias),
@@ -316,16 +356,22 @@ class RestfulRouter:
 
         return self
 
-    def default(self, dependency: Dependency) -> Self:
+    def default(self) -> Self:
         return (
-            self.with_create_one_endpoint(dependency)
-            .with_create_many_endpoint(dependency)
-            .with_read_one_endpoint(dependency)
-            .with_read_all_endpoint(dependency)
-            .with_update_one_endpoint(dependency)
-            .with_update_many_endpoint(dependency)
-            .with_delete_one_endpoint(dependency)
+            self.with_create_one_endpoint()
+            .with_create_many_endpoint()
+            .with_read_one_endpoint()
+            .with_read_all_endpoint()
+            .with_update_one_endpoint()
+            .with_update_many_endpoint()
+            .with_delete_one_endpoint()
         )
 
     def build(self) -> APIRouter:
         return self.router
+
+    def _resolve(self, dependency: Dependency | None) -> type[RestfulService]:
+        resolved = dependency or self.dependency
+        assert resolved, "One of default or endpoint dependency must be specified"
+
+        return resolved.as_dependable()
