@@ -1,12 +1,20 @@
 from abc import ABC, abstractmethod
+from collections.abc import Callable, Iterable
 from dataclasses import dataclass
 from functools import cached_property
-from typing import Any, Callable, Iterable, List
+from typing import Any
 
 from pydantic import BaseModel, create_model
 
 from apexdevkit.fastapi.name import RestfulName
 from apexdevkit.fluent import FluentDict
+from apexdevkit.http import JsonDict
+from apexdevkit.value import Value
+
+
+class AggregationResult(BaseModel):
+    field: str
+    aggregation: Value
 
 
 class SchemaFields(ABC):
@@ -19,6 +27,15 @@ class SchemaFields(ABC):
     def editable(self) -> FluentDict[type]:
         return self.readable().drop("id")
 
+    def filters(self) -> FluentDict[type]:
+        return JsonDict()
+
+    def aggregation_filters(self) -> FluentDict[type]:
+        return JsonDict()
+
+    def aggregation_result(self) -> FluentDict[type]:
+        return JsonDict().with_a(count=int).and_a(sums=list[AggregationResult])
+
     @abstractmethod
     def readable(self) -> FluentDict[type]:  # pragma: no cover
         pass
@@ -37,12 +54,15 @@ class RestfulSchema:
         update_many_item = self._schema_for(
             "UpdateManyItem", self.fields.editable().merge(self.fields.id())
         )
+        self._schema_for("Filter", self.fields.filters())
+        self._schema_for("Aggregation", self.fields.aggregation_filters())
+        self._schema_for("AggregationResult", self.fields.aggregation_result())
 
         self._schema_for("Item", {self.name.singular: schema})
-        self._schema_for("Collection", {self.name.plural: List[schema], "count": int})
-        self._schema_for("CreateMany", {self.name.plural: List[create_schema]})
-        self._schema_for("UpdateMany", {self.name.plural: List[update_many_item]})
-        self._schema_for("ReplaceMany", {self.name.plural: List[replace_schema]})
+        self._schema_for("Collection", {self.name.plural: list[schema], "count": int})
+        self._schema_for("CreateMany", {self.name.plural: list[create_schema]})
+        self._schema_for("UpdateMany", {self.name.plural: list[update_many_item]})
+        self._schema_for("ReplaceMany", {self.name.plural: list[replace_schema]})
 
     def _schema_for(self, action: str, fields: dict[str, Any]) -> type[BaseModel]:
         if action not in self.schemas:
@@ -129,6 +149,31 @@ class RestfulSchema:
 
         return _
 
+    def for_filters(self) -> Callable[[BaseModel], dict[str, Any]]:
+        schema = self.schemas["Filter"]
+
+        def _(request: schema) -> dict[str, Any]:
+            return request.model_dump()
+
+        return _
+
+    def for_aggregation(self) -> Callable[[BaseModel], dict[str, Any]]:
+        schema = self.schemas["Aggregation"]
+
+        def _(request: schema) -> dict[str, Any]:
+            return request.model_dump()
+
+        return _
+
+    def for_aggregation_result(self) -> type[BaseModel]:
+        return self._schema_for(
+            "AggregationResultResponse",
+            FluentDict[type]()
+            .with_a(status=str)
+            .and_a(code=int)
+            .and_a(aggregations=self.schemas["AggregationResult"]),
+        )
+
 
 @dataclass(frozen=True)
 class Schema:
