@@ -1,18 +1,19 @@
 from __future__ import annotations
 
-from abc import ABC
+from collections.abc import Iterable, Mapping
 from dataclasses import dataclass
-from typing import Any, Dict, Generic, Iterable, TypeVar
+from functools import cached_property
+from typing import Any, Generic, TypeVar
 
 from apexdevkit.formatter import Formatter
-from apexdevkit.repository.decorator import BatchRepositoryDecorator
+from apexdevkit.repository.decorator import BruteForceBatch
 from apexdevkit.repository.interface import Repository
 
-RawItem = dict[str, Any]
+RawItem = Mapping[str, Any]
 RawCollection = Iterable[RawItem]
 
 
-class _RawItemWithId(Dict[str, Any]):
+class _RawItemWithId(dict[str, Any]):
     def __post_init__(self) -> None:
         assert "id" in self
 
@@ -20,7 +21,7 @@ class _RawItemWithId(Dict[str, Any]):
 RawCollectionWithId = Iterable[_RawItemWithId]
 
 
-class RestfulService(ABC):  # pragma: no cover
+class RestfulService:  # pragma: no cover
     def create_one(self, item: RawItem) -> RawItem:
         raise NotImplementedError(self.create_one.__name__)
 
@@ -33,6 +34,12 @@ class RestfulService(ABC):  # pragma: no cover
     def read_many(self, **params: Any) -> RawCollection:
         raise NotImplementedError(self.read_many.__name__)
 
+    def filter_with(self, options: RawItem) -> RawCollection:
+        raise NotImplementedError(self.filter_with.__name__)
+
+    def aggregation_with(self, options: RawItem) -> RawItem:
+        raise NotImplementedError(self.filter_with.__name__)
+
     def read_all(self) -> RawCollection:
         raise NotImplementedError(self.read_all.__name__)
 
@@ -56,38 +63,22 @@ ItemT = TypeVar("ItemT")
 
 
 @dataclass(frozen=True)
-class RestfulRepositoryBuilder(Generic[ItemT]):
-    formatter: Formatter[dict[str, Any], ItemT] | None = None
-    repository: Repository[ItemT] | None = None
-
-    def with_formatter(
-        self, formatter: Formatter[dict[str, Any], ItemT]
-    ) -> RestfulRepositoryBuilder[ItemT]:
-        return RestfulRepositoryBuilder[ItemT](formatter, self.repository)
-
-    def with_repository(
-        self, repository: Repository[ItemT]
-    ) -> RestfulRepositoryBuilder[ItemT]:
-        return RestfulRepositoryBuilder[ItemT](self.formatter, repository)
-
-    def build(self) -> RestfulService:
-        if self.formatter is None or self.repository is None:
-            raise RuntimeError("Formatter or repository not provided.")
-        return _RestfulRepository(self.formatter, self.repository)
-
-
-@dataclass(frozen=True)
-class _RestfulRepository(RestfulService, Generic[ItemT]):
-    formatter: Formatter[dict[str, Any], ItemT]
+class RestfulRepository(RestfulService, Generic[ItemT]):
     repository: Repository[ItemT]
 
+    formatter: Formatter[Mapping[str, Any], ItemT]
+
+    @cached_property
+    def _batch(self) -> BruteForceBatch[ItemT]:
+        return BruteForceBatch(self.repository)
+
     def create_one(self, item: RawItem) -> RawItem:
         return self.formatter.dump(self.repository.create(self.formatter.load(item)))
 
     def create_many(self, items: RawCollection) -> RawCollection:
         return [
             self.formatter.dump(item)
-            for item in BatchRepositoryDecorator(self.repository).create_many(
+            for item in self._batch.create_many(
                 [self.formatter.load(fields) for fields in items]
             )
         ]
@@ -99,7 +90,7 @@ class _RestfulRepository(RestfulService, Generic[ItemT]):
         return [self.formatter.dump(item) for item in self.repository]
 
     def update_one(self, item_id: str, **with_fields: Any) -> RawItem:
-        data = self.formatter.dump(self.repository.read(item_id))
+        data = dict(self.formatter.dump(self.repository.read(item_id)))
         data.update(**with_fields)
 
         self.repository.update(self.formatter.load(data))
@@ -109,12 +100,12 @@ class _RestfulRepository(RestfulService, Generic[ItemT]):
     def update_many(self, items: RawCollectionWithId) -> RawCollection:
         updates = []
         for fields in items:
-            data = self.formatter.dump(self.repository.read(fields["id"]))
+            data = dict(self.formatter.dump(self.repository.read(fields["id"])))
             data.update(**fields)
 
             updates.append(self.formatter.load(data))
 
-        BatchRepositoryDecorator(self.repository).update_many(updates)
+        self._batch.update_many(updates)
 
         return [self.formatter.dump(item) for item in updates]
 
@@ -124,9 +115,7 @@ class _RestfulRepository(RestfulService, Generic[ItemT]):
         return item
 
     def replace_many(self, items: RawCollection) -> RawCollection:
-        BatchRepositoryDecorator(self.repository).update_many(
-            [self.formatter.load(item) for item in items]
-        )
+        self._batch.update_many([self.formatter.load(item) for item in items])
 
         return items
 
