@@ -1,11 +1,10 @@
 from __future__ import annotations
 
 import pickle
+from collections.abc import Mapping
 from copy import deepcopy
 from dataclasses import asdict, dataclass, field, fields, is_dataclass
-from typing import Any, Generic, Protocol, Self, TypeVar, get_args
-
-from typing_extensions import get_type_hints
+from typing import Any, Generic, Protocol, Self, TypeVar, get_args, get_type_hints
 
 from apexdevkit.fluent import FluentDict
 from apexdevkit.value import Value
@@ -23,6 +22,37 @@ class Formatter(Protocol[_SourceT, _TargetT]):  # pragma: no cover
         pass
 
 
+@dataclass(frozen=True)
+class AliasFormatter(Formatter[Mapping[str, Any], _TargetT]):
+    inner: Formatter[Mapping[str, Any], _TargetT]
+
+    alias: AliasMapping
+
+    def load(self, source: Mapping[str, Any]) -> _TargetT:
+        return self.inner.load(self.alias.reverse().translate(source))
+
+    def dump(self, target: _TargetT) -> Mapping[str, Any]:
+        return self.alias.translate(self.inner.dump(target))
+
+
+@dataclass(frozen=True)
+class AliasMapping:
+    alias: Mapping[str, str]
+
+    @classmethod
+    def parse(cls, **alias: str) -> AliasMapping:
+        return cls(alias)
+
+    def reverse(self) -> AliasMapping:
+        return AliasMapping({value: key for key, value in self.alias.items()})
+
+    def translate(self, mapping: Mapping[str, Any]) -> Mapping[str, Any]:
+        return {self.value_of(key): value for key, value in mapping.items()}
+
+    def value_of(self, key: str) -> str:
+        return self.alias.get(key, key)
+
+
 class PickleFormatter(Generic[_ItemT]):
     def dump(self, item: _ItemT) -> bytes:
         return pickle.dumps(item)
@@ -55,7 +85,7 @@ class DataclassFormatter(Generic[_TargetT]):
 
         return self
 
-    def load(self, raw: dict[str, Any]) -> _TargetT:
+    def load(self, raw: Mapping[str, Any]) -> _TargetT:
         raw = FluentDict[Any](deepcopy(raw)).select(
             *self.resource.__annotations__.keys()
         )
@@ -65,7 +95,7 @@ class DataclassFormatter(Generic[_TargetT]):
             key_type = types[key.name]
             if key.name not in raw:
                 continue
-            elif key.name in self.sub_formatters.keys():
+            if key.name in self.sub_formatters:
                 raw[key.name] = (
                     self.sub_formatters[key.name].load(raw.pop(key.name))
                     if raw[key.name]
@@ -82,13 +112,13 @@ class DataclassFormatter(Generic[_TargetT]):
 
         return self.resource(**raw)
 
-    def dump(self, item: _TargetT) -> dict[str, Any]:
+    def dump(self, item: _TargetT) -> Mapping[str, Any]:
         return asdict(item)  # type: ignore
 
 
 class ValueFormatter:
-    def load(self, raw: dict[str, Any]) -> Value:
+    def load(self, raw: Mapping[str, Any]) -> Value:
         return DataclassFormatter(Value).load(raw)
 
-    def dump(self, value: Value) -> dict[str, Any]:
+    def dump(self, value: Value) -> Mapping[str, Any]:
         return DataclassFormatter(Value).dump(value)
