@@ -1,8 +1,10 @@
 from __future__ import annotations
 
+from collections import defaultdict
+from collections.abc import Callable, Iterable, Iterator, MutableMapping
 from contextlib import suppress
 from dataclasses import dataclass, field
-from typing import Any, Callable, Generic, Iterable, Iterator, Protocol, Self
+from typing import Any, Generic, Protocol, Self
 
 from apexdevkit.error import DoesNotExistError, ExistsError
 from apexdevkit.formatter import Formatter, PickleFormatter
@@ -19,25 +21,14 @@ class InMemoryRepository(Generic[ItemT]):
     keys: list[_KeyFunction[ItemT]] = field(default_factory=list)
     seeds: frozenset[ItemT] = field(default_factory=frozenset)
 
-    def with_namespace(self, value: str) -> InMemoryRepository[ItemT]:
-        return InMemoryRepository[ItemT](
-            store=StoreNamespace(value, self.store),
-            keys=self.keys,
-            seeds=self.seeds,
-        )
-
     def with_store(self, value: KeyValueStore[ItemT]) -> InMemoryRepository[ItemT]:
-        return InMemoryRepository[ItemT](
-            store=value,
-            keys=self.keys,
-            seeds=self.seeds,
-        )
+        return InMemoryRepository(store=value, keys=self.keys, seeds=self.seeds)
 
     def and_key(self, function: _KeyFunction[ItemT]) -> InMemoryRepository[ItemT]:
         return self.with_key(function)
 
     def with_key(self, function: _KeyFunction[ItemT]) -> InMemoryRepository[ItemT]:
-        return InMemoryRepository[ItemT](
+        return InMemoryRepository(
             store=self.store,
             keys=[*self.keys, function],
             seeds=self.seeds,
@@ -112,29 +103,18 @@ class InMemoryByteStore(Generic[ItemT]):
         for raw in self.items.values():
             yield self.formatter.load(raw)
 
+    @dataclass
+    class Cache:
+        items: MutableMapping[str, InMemoryByteStore[Any]] = field(init=False)
 
-@dataclass
-class StoreNamespace(Generic[ItemT]):
-    name: str
-    inner: KeyValueStore[ItemT]
+        def __post_init__(self) -> None:
+            self.items = defaultdict(InMemoryByteStore)
 
-    def count(self) -> int:
-        return self.inner.count()
+        def store_for(self, name: str) -> InMemoryByteStore[Any]:
+            return self.items[name]
 
-    def values(self) -> Iterable[ItemT]:
-        return self.inner.values()
-
-    def set(self, key: str, value: ItemT) -> None:
-        self.inner.set(self._expand(key), value)
-
-    def get(self, key: str) -> ItemT:
-        return self.inner.get(self._expand(key))
-
-    def drop(self, key: str) -> None:
-        self.inner.drop(self._expand(key))
-
-    def _expand(self, key: str) -> str:
-        return "-".join([self.name, key])
+        def clear(self) -> None:
+            self.items.clear()
 
 
 @dataclass
@@ -142,7 +122,7 @@ class _SingleKeyRepository(RepositoryBase[ItemT]):
     store: KeyValueStore[ItemT]
     pk: _KeyFunction[ItemT]
 
-    def bind(self, **kwargs: Any) -> Self:  # pragma: no cover
+    def bind(self, **_: Any) -> Self:  # pragma: no cover
         return self
 
     def create(self, item: ItemT) -> ItemT:
@@ -166,14 +146,14 @@ class _SingleKeyRepository(RepositoryBase[ItemT]):
     def delete(self, item_id: str) -> None:
         try:
             self.store.drop(item_id)
-        except KeyError:
-            raise DoesNotExistError(item_id)
+        except KeyError as e:
+            raise DoesNotExistError(item_id) from e
 
     def read(self, item_id: str) -> ItemT:
         try:
             return self.store.get(item_id)
-        except KeyError:
-            raise DoesNotExistError(item_id)
+        except KeyError as e:
+            raise DoesNotExistError(item_id) from e
 
     def __iter__(self) -> Iterator[ItemT]:
         return iter(self.store.values())
@@ -188,7 +168,7 @@ class _ManyKeyRepository(RepositoryBase[ItemT]):
 
     keys: list[_KeyFunction[ItemT]] = field(default_factory=list)
 
-    def bind(self, **kwargs: Any) -> Self:  # pragma: no cover
+    def bind(self, **_: Any) -> Self:  # pragma: no cover
         return self
 
     def create(self, item: ItemT) -> ItemT:
