@@ -1,7 +1,7 @@
 from __future__ import annotations
 
-from collections.abc import Iterable
-from typing import Any, Iterator, Protocol, TypeVar
+from collections.abc import Iterable, Iterator
+from typing import Any, Protocol, TypeVar
 
 ItemT = TypeVar("ItemT")
 
