@@ -1,9 +1,10 @@
 from __future__ import annotations
 
+from collections.abc import Iterable, Iterator, Mapping
 from dataclasses import dataclass
-from typing import Any, Generic, Iterable, Iterator, TypeVar
+from typing import Any, Generic, TypeVar
 
-from pymssql.exceptions import DatabaseError
+from pymssql.exceptions import DatabaseError, OperationalError
 
 from apexdevkit.error import DoesNotExistError, ExistsError
 from apexdevkit.formatter import Formatter
@@ -27,8 +28,8 @@ class MsSqlRepository(RepositoryBase[ItemT]):
 
         try:
             return int(raw["n_items"])
-        except KeyError:
-            raise UnknownError(raw)
+        except KeyError as e:
+            raise UnknownError(raw) from e
 
     def delete(self, item_id: str) -> None:
         self.db.execute(self.table.delete(item_id)).fetch_none()
@@ -40,15 +41,18 @@ class MsSqlRepository(RepositoryBase[ItemT]):
         try:
             return self.table.load(self.db.execute(self.table.insert(item)).fetch_one())
         except DatabaseError as e:
-            e = MssqlException(e)
+            if MssqlException(e).is_duplication():
+                raise self.table.exists(item) from e
 
-            if e.is_duplication():
-                raise self.table.exists(item)
-
-            raise UnknownError(e.message)
+            raise UnknownError(MssqlException(e).message) from e
 
     def read(self, item_id: str) -> ItemT:
-        raw = self.db.execute(self.table.select(item_id)).fetch_one()
+        try:
+            raw = self.db.execute(self.table.select(item_id)).fetch_one()
+        except OperationalError as e:
+            if "Conversion failed" in str(e):
+                raise DoesNotExistError(item_id) from e
+            raise e
 
         if not raw:
             raise DoesNotExistError(item_id)
@@ -81,7 +85,7 @@ class SqlTable(Generic[ItemT]):  # pragma: no cover
     def update(self, item: ItemT) -> DatabaseCommand:
         raise NotImplementedError
 
-    def load(self, data: dict[str, Any]) -> ItemT:
+    def load(self, data: Mapping[str, Any]) -> ItemT:
         raise NotImplementedError
 
     def exists(self, duplicate: ItemT) -> ExistsError:
@@ -143,8 +147,9 @@ class MsSqlTableBuilder(Generic[ItemT]):
     username: str | None = None
     schema: str | None = None
     table: str | None = None
-    formatter: Formatter[dict[str, Any], ItemT] | None = None
+    formatter: Formatter[Mapping[str, Any], ItemT] | None = None
     fields: list[_SqlField] | None = None
+    custom_filters: list[str] | None = None
 
     def with_username(self, value: str) -> MsSqlTableBuilder[ItemT]:
         return MsSqlTableBuilder[ItemT](
@@ -153,6 +158,7 @@ class MsSqlTableBuilder(Generic[ItemT]):
             self.table,
             self.formatter,
             self.fields,
+            self.custom_filters,
         )
 
     def with_schema(self, value: str) -> MsSqlTableBuilder[ItemT]:
@@ -162,6 +168,7 @@ class MsSqlTableBuilder(Generic[ItemT]):
             self.table,
             self.formatter,
             self.fields,
+            self.custom_filters,
         )
 
     def with_table(self, value: str) -> MsSqlTableBuilder[ItemT]:
@@ -171,10 +178,11 @@ class MsSqlTableBuilder(Generic[ItemT]):
             value,
             self.formatter,
             self.fields,
+            self.custom_filters,
         )
 
     def with_formatter(
-        self, value: Formatter[dict[str, Any], ItemT]
+        self, value: Formatter[Mapping[str, Any], ItemT]
     ) -> MsSqlTableBuilder[ItemT]:
         return MsSqlTableBuilder[ItemT](
             self.username,
@@ -182,6 +190,7 @@ class MsSqlTableBuilder(Generic[ItemT]):
             self.table,
             value,
             self.fields,
+            self.custom_filters,
         )
 
     def with_fields(self, value: Iterable[_SqlField]) -> MsSqlTableBuilder[ItemT]:
@@ -207,17 +216,32 @@ class MsSqlTableBuilder(Generic[ItemT]):
             self.table,
             self.formatter,
             key_list,
+            self.custom_filters,
+        )
+
+    def with_custom_filters(self, filters: Iterable[str]) -> MsSqlTableBuilder[ItemT]:
+        return MsSqlTableBuilder[ItemT](
+            self.username,
+            self.schema,
+            self.table,
+            self.formatter,
+            self.fields,
+            list(filters),
         )
 
     def build(self) -> SqlTable[ItemT]:
         if not self.schema or not self.table or not self.formatter or not self.fields:
             raise ValueError("Cannot build sql table.")
 
+        field_manager = SqlFieldManager.Builder().with_fields(self.fields)
+        if self.custom_filters and len(self.custom_filters) > 0:
+            field_manager = field_manager.with_custom_filters(self.custom_filters)
+
         return DefaultSqlTable(
             self.schema,
             self.table,
             self.formatter,
-            SqlFieldManager.Builder().with_fields(self.fields).for_mssql().build(),
+            field_manager.for_mssql().build(),
             self.username,
         )
 
@@ -226,7 +250,7 @@ class MsSqlTableBuilder(Generic[ItemT]):
 class DefaultSqlTable(SqlTable[ItemT]):
     schema: str
     table: str
-    formatter: Formatter[dict[str, Any], ItemT]
+    formatter: Formatter[Mapping[str, Any], ItemT]
     fields: SqlFieldManager
     username: str | None = None
 
@@ -246,19 +270,32 @@ class DefaultSqlTable(SqlTable[ItemT]):
         placeholders = ", ".join(
             [f"%({key.name})s" for key in self.fields if key.include_in_insert]
         )
-        output = ", ".join(
-            ["INSERTED." + field.name + " AS " + field.name for field in self.fields]
-        )
+        try:
+            _ = self.fields.id
+            output = ", ".join(
+                ["[" + field.name + "] AS " + field.name for field in self.fields]
+            )
+            where_statement = f"""
+                FROM [{self.schema}].[{self.table}]
+                {self.fields.where_statement(include_id=True, read_id=True)}
+            """
+        except ValueError:
+            output = ", ".join(
+                ["%(" + field.name + ")s AS " + field.name for field in self.fields]
+            )
+            where_statement = ""
 
         return DatabaseCommand(f"""
             {self._user_check}
             INSERT INTO [{self.schema}].[{self.table}] (
                 {columns}
-            ) OUTPUT
-                {output}
+            )
             VALUES (
                 {placeholders}
-            )
+            );
+            SELECT
+                {output}
+            {where_statement}
             REVERT
         """).with_data(self.fields.with_fixed(self.formatter.dump(item)))
 
@@ -268,7 +305,7 @@ class DefaultSqlTable(SqlTable[ItemT]):
         return DatabaseCommand(f"""
             {self._user_check}
             SELECT
-                {columns} 
+                {columns}
             FROM [{self.schema}].[{self.table}]
             {self.fields.where_statement(include_id=True)}
             REVERT
@@ -323,18 +360,17 @@ class DefaultSqlTable(SqlTable[ItemT]):
             REVERT
         """).with_data(self.fields.with_fixed({}))
 
-    def load(self, data: dict[str, Any]) -> ItemT:
+    def load(self, data: Mapping[str, Any]) -> ItemT:
         return self.formatter.load(data)
 
     def exists(self, duplicate: ItemT) -> ExistsError:
         raw = self.formatter.dump(duplicate)
         return ExistsError(duplicate).with_duplicate(
-            lambda i: f"{self.fields.id}<{raw[self.fields.id]}>"
+            lambda _: f"{self.fields.id}<{raw[self.fields.id]}>"
         )
 
     @property
     def _user_check(self) -> str:
         if self.username is not None:
             return f"EXECUTE AS USER = '{self.username}'"
-        else:
-            return ""
+        return ""
