@@ -1,8 +1,9 @@
 from __future__ import annotations
 
+from collections.abc import Iterable, Iterator, Mapping
 from dataclasses import dataclass
 from sqlite3 import IntegrityError
-from typing import Any, Generic, Iterable, Iterator
+from typing import Any, Generic
 
 from apexdevkit.error import DoesNotExistError, ExistsError
 from apexdevkit.formatter import Formatter
@@ -28,8 +29,8 @@ class SqliteRepository(RepositoryBase[ItemT]):
 
         try:
             return int(raw["n_items"])
-        except KeyError:
-            raise UnknownError(raw)
+        except KeyError as e:
+            raise UnknownError(raw) from e
 
     def create(self, item: ItemT) -> ItemT:
         try:
@@ -60,7 +61,7 @@ class SqliteRepository(RepositoryBase[ItemT]):
 
 
 class SqlTable(Generic[ItemT]):  # pragma: no cover
-    def bind(self, **kwargs: Any) -> SqlTable[ItemT]:
+    def bind(self, **_: Any) -> SqlTable[ItemT]:
         return self
 
     def count_all(self) -> DatabaseCommand:
@@ -87,31 +88,42 @@ class SqlTable(Generic[ItemT]):  # pragma: no cover
     def delete_all(self) -> DatabaseCommand:
         raise NotImplementedError
 
-    def load(self, data: dict[str, Any]) -> ItemT:
+    def load(self, data: Mapping[str, Any]) -> ItemT:
         raise NotImplementedError
 
     def duplicate(self, item: ItemT) -> ExistsError:
-        return ExistsError(item).with_duplicate(lambda i: "Unknown")
+        return ExistsError(item).with_duplicate(lambda _: "Unknown")
 
 
 @dataclass
 class UnknownError(Exception):
-    raw: dict[str, Any]
+    raw: Mapping[str, Any]
 
 
 @dataclass(frozen=True)
 class SqliteTableBuilder(Generic[ItemT]):
     table_name: str | None = None
-    formatter: Formatter[dict[str, Any], ItemT] | None = None
+    formatter: Formatter[Mapping[str, Any], ItemT] | None = None
     fields: list[_SqlField] | None = None
+    custom_filters: list[str] | None = None
 
     def with_name(self, value: str) -> SqliteTableBuilder[ItemT]:
-        return SqliteTableBuilder[ItemT](value, self.formatter, self.fields)
+        return SqliteTableBuilder[ItemT](
+            value,
+            self.formatter,
+            self.fields,
+            self.custom_filters,
+        )
 
     def with_formatter(
-        self, value: Formatter[dict[str, Any], ItemT]
+        self, value: Formatter[Mapping[str, Any], ItemT]
     ) -> SqliteTableBuilder[ItemT]:
-        return SqliteTableBuilder[ItemT](self.table_name, value, self.fields)
+        return SqliteTableBuilder[ItemT](
+            self.table_name,
+            value,
+            self.fields,
+            self.custom_filters,
+        )
 
     def with_fields(self, value: Iterable[_SqlField]) -> SqliteTableBuilder[ItemT]:
         key_list = list(value)
@@ -134,23 +146,36 @@ class SqliteTableBuilder(Generic[ItemT]):
             self.table_name,
             self.formatter,
             key_list,
+            self.custom_filters,
+        )
+
+    def with_custom_filters(self, filters: Iterable[str]) -> SqliteTableBuilder[ItemT]:
+        return SqliteTableBuilder[ItemT](
+            self.table_name,
+            self.formatter,
+            self.fields,
+            list(filters),
         )
 
     def build(self) -> SqlTable[ItemT]:
         if not self.table_name or not self.formatter or not self.fields:
             raise ValueError("Parameter missing.")
 
+        field_manager = SqlFieldManager.Builder().with_fields(self.fields)
+        if self.custom_filters and len(self.custom_filters) > 0:
+            field_manager = field_manager.with_custom_filters(self.custom_filters)
+
         return _DefaultSqlTable(
             self.table_name,
             self.formatter,
-            SqlFieldManager.Builder().with_fields(self.fields).for_sqlite().build(),
+            field_manager.for_sqlite().build(),
         )
 
 
 @dataclass(frozen=True)
 class _DefaultSqlTable(SqlTable[ItemT]):
     table_name: str
-    formatter: Formatter[dict[str, Any], ItemT]
+    formatter: Formatter[Mapping[str, Any], ItemT]
     fields: SqlFieldManager
 
     def count_all(self) -> DatabaseCommand:
@@ -183,7 +208,7 @@ class _DefaultSqlTable(SqlTable[ItemT]):
 
         return DatabaseCommand(f"""
             SELECT
-                {columns} 
+                {columns}
             FROM {self.table_name.upper()}
             {self.fields.where_statement(include_id=True)};
         """).with_data(self.fields.with_fixed({self.fields.id: item_id}))
@@ -198,7 +223,7 @@ class _DefaultSqlTable(SqlTable[ItemT]):
 
         return DatabaseCommand(f"""
             SELECT
-                {columns} 
+                {columns}
             FROM {self.table_name.upper()}
             WHERE {duplicates};
         """).with_data({key: raw[key] for key in raw if key in self.fields.composite})
@@ -244,13 +269,13 @@ class _DefaultSqlTable(SqlTable[ItemT]):
             {self.fields.where_statement(include_id=False)};
         """).with_data(self.fields.with_fixed({}))
 
-    def load(self, data: dict[str, Any]) -> ItemT:
+    def load(self, data: Mapping[str, Any]) -> ItemT:
         return self.formatter.load(data)
 
     def duplicate(self, item: ItemT) -> ExistsError:
         raw = self.formatter.dump(item)
         return ExistsError(item).with_duplicate(
-            lambda i: ",".join(
+            lambda _: ",".join(
                 [f"{key}<{raw[key]}>" for key in raw if key in self.fields.composite]
             )
         )
